using System;
using System.Collections.Generic;
using System.IO;
using Vintagestory.API.Common;
using Vintagestory.API.Server;

namespace AnnouncerMod
{
    public class ModRestartAnnouncement : ModSystem
    {
        ICoreServerAPI api;
        bool announcedToday = false;
        DateTime lastAnnouncementDate;

        List<Announcement> announcements = new List<Announcement>();

        public override void StartServerSide(ICoreServerAPI api)
        {
            this.api = api;

            api.Event.RegisterGameTickListener(OnGameTick, 1000);

            lastAnnouncementDate = DateTime.Now;

            LoadConfig();
        }

        private string GetConfigPath()
        {
            string appDataPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
            string configDirectory = Path.Combine(appDataPath, "vintagestorydata", "ModConfig");
            Directory.CreateDirectory(configDirectory);
            return Path.Combine(configDirectory, "AnnouncerConfig.json");
        }

        private void LoadConfig()
        {
            string configPath = GetConfigPath();

            if (File.Exists(configPath))
            {
                try
                {
                    string json = File.ReadAllText(configPath);
                    announcements = System.Text.Json.JsonSerializer.Deserialize<List<Announcement>>(json);

                    if (announcements == null || announcements.Count == 0)
                    {
                        announcements = new List<Announcement>
                        {
                            new Announcement { Hour = 6, Minute = 0, Message = "placeholder anouncement(u should totally set this up it'd be v cool of you to use this)" }
                        };
                        SaveConfig();
                    }
                }
                catch (Exception e)
                {
                    api.Logger.Error("Failed to load Announcer config: " + e.Message);
                }
            }
            else
            {
                announcements = new List<Announcement>
                {
                    new Announcement { Hour = 6, Minute = 0, Message = "placeholder anouncement(u should totally set this up it'd be v cool of you to use this)" }
                };
                SaveConfig();
            }
        }

        private void SaveConfig()
        {
            string configPath = GetConfigPath();

            try
            {
                string json = System.Text.Json.JsonSerializer.Serialize(announcements, new System.Text.Json.JsonSerializerOptions { WriteIndented = true });
                File.WriteAllText(configPath, json);
            }
            catch (Exception e)
            {
                api.Logger.Error("Failed to save Announcer config: " + e.Message);
            }
        }

        private void OnGameTick(float deltaTime)
        {
            DateTime now = DateTime.Now;

            if (now.Date > lastAnnouncementDate)
            {
                announcedToday = false;
                lastAnnouncementDate = now.Date;
            }

            foreach (var announcement in announcements)
            {
                DateTime nextAnnouncementTime = new DateTime(now.Year, now.Month, now.Day, announcement.Hour, announcement.Minute, 0);

                if (nextAnnouncementTime < now)
                {
                    nextAnnouncementTime = nextAnnouncementTime.AddDays(1);
                }

                TimeSpan timeUntilNextAnnouncement = nextAnnouncementTime - now;

                if (timeUntilNextAnnouncement.TotalMilliseconds <= deltaTime * 1000)
                {
                    api.InjectConsole(announcement.Message);

                    announcedToday = true;
                }
            }
        }

        private class Announcement
        {
            public int Hour { get; set; }
            public int Minute { get; set; }
            public string Message { get; set; }
        }
    }
}
